#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-

##########################################################################################
# pihole special agent
#
# Author: Florian Dille
# Date: 12/2021
#
# Note:
#   this is the part which creates the WATO Element to configure the special agent
#   filename: doesn't matter
#   location: <site>/local/share/check_mk/web/plugins/wato/watoRuleset_<myCheckName>.py
# 
##########################################################################################

from cmk.gui.i18n import _

# import required to register agent
from cmk.gui.plugins.wato import (
    rulespec_registry,
    HostRulespec,
    CheckParameterRulespecWithItem,
    RulespecGroupCheckParametersOperatingSystem
)

from cmk.gui.valuespec import (
    FixedValue,
    TextInput,
    Age,
    ListOfStrings,
    DropdownChoice, 
)

# import structure where special agent will be registered
try:
    from cmk.gui.plugins.wato.datasource_programs import RulespecGroupDatasourcePrograms # breaking change for 2.1
except ImportError:
    from cmk.gui.plugins.wato.special_agents.common import RulespecGroupDatasourcePrograms # for versions prior to 2.1

##########################################################################################
##### Special Agent Rule
##########################################################################################
##### param definition

_common_params_host = (
    ("host", TextAscii(title=_("Pi-hole Host"), allow_empty=False, help = "Hostname or IP Address of your Pi-hole"))
)

_common_params_port = (
    ("port", Integer(title=_("Pi-hole Host Port"), help="Port via which the Api is reachable", default_value=80))
)

_common_params_ignoressl = (
    ("ignoressl", FixedValue(value = True, totext = 'ignoring ssl certificate errors', title=_("ignore ssl error"),help="agent ignores ssl (validation) errors, e.g. in case of a self-signed certificate"))
)

_common_params_usessl = (
    ("usessl", FixedValue(value = True, totext = 'Agent will use https instead', title=_("use https"), help = "use https protocol instead of Pi-hole http default"))
)

_common_params_token = (
    ("token", Password(title=_("Pi-hole API Token"), help = "if your admin has a password set enter the related token, else leave empty", allow_empty=True))
)


##########################################################################################
##### wato rule definition

# Some WATO form definition, to ask user for params - pihole_stats
def _valuespec_special_agent_pihole():
    return Dictionary(
        title = _("Pi-hole API Special Agent"),
        help = _("Pi-hole Special Agent which talks to the Pi-hole API"),
        elements = [
           _common_params_host,
           _common_params_port,
           _common_params_usessl,
           _common_params_ignoressl,
           _common_params_token,
        ],
        optional_keys = ['usessl','ignoressl'],
    )


##########################################################################################
##### register rules

# this registers the Special Agent - pihole_stats
rulespec_registry.register(
    HostRulespec(
        group=RulespecGroupDatasourcePrograms,
        # IMPORTANT, name must follow special_agents:<name>, 
        # where filename of our special agent located in path local/share/check_mk/agents/special/ is  agent_<name>
        name="special_agents:pihole",
        valuespec=_valuespec_special_agent_pihole,
    ))


##########################################################################################
##### Check rule
##########################################################################################

def _item_valuespec_pihole():
    return TextInput(title=_("Item name"))

#########################################################################################
##### param definition 

_options_gravity = (
    "gravity",
    Dictionary(
        title = _("Gravity Last Update"),
        help = "Define a age for the last Gravity update",
        elements = [
            ("age",Age(title=_("last check age"), help = "set the maximum age the last update should have")),
            ("state", MonitoringState(title=_("Service state if last update to old"), help = "set the state the service should have if the age is to old, default is WARN", default_value=1)),
        ]
    )
)

_options_dbfilesize = (
    "dbfilesize",
    Dictionary(
        title = _("DB File Size"),
        help = "Size of the pi-hole db file",
        elements = [
            ("levels_lower", Tuple(title=_("Lower file size levels"), help = "minimum size the file should have, 0 for do not apply", elements=[
                (Filesize(title=_("Size warn below"),)),
                (Filesize(title=_("Size crit below"),)),
            ])),
            ("levels_upper", Tuple(title=_("Upper file size levels"), help = "maximum size the file should have, 0 for do not apply", elements=[
                (Filesize(title=_("Size warn above"),)),
                (Filesize(title=_("Size crit above"),)),
            ])),
        ]
    )
)

_options_state = (
    "state",
    Dictionary(
        title = _("Pi-hole State"),
        help = "Pi-hole State",
        elements = [
            (
                "dst",
                DropdownChoice(
                    title=_("desired Pi-hole state"),
                    help=_("select the state Pi-hole should have"),
                    default_value = None,
                    sorted = False,
                    choices=[
                        (None, _('--- select ---')),
                        (True, _("Enable")),
                        (False, _("Disable")),
                    ],
                )
            ),
            ("state", MonitoringState(title=_("Service state if pi-hole state != desired state"), help = "set the state the Service will have if the Pi-hole state is not as expected", default_value=2)),
        ]
    )
)

_options_messages = (
    'messages',
    Dictionary(
        title = _('Status Messages'),
        help = "define some parameters for possible Pi-hole messages",
        elements = [
            ("nohtml", FixedValue(value = True, totext = 'html output disabled', title=_("plaintext check output (no HTML)"), help = "if you prefer non HTML output or have HTML output disabled. The non HTML output format still need some further improvement")),
            ("default_state", MonitoringState(title=_("default state (for message types not further defined below)"), help = "the default state effect for messages not explicitly defined as OK, WARN or CRIT", default_value=2)),
            (
                "msg_ok",
                ListOfStrings(
                    title=_("always OK message types"),
                    help = "Message Types which are allways OK (max messages will count these nevertheless)",
                    orientation='horizontal',
                    allow_empty=False,
                    valuespec=TextInput(allow_empty=True, regex='[a-zA-Z0-9\.]'),
                )
            ),
            (
                "msg_warn",
                ListOfStrings(
                    title=_("WARN message types"),
                    help = "Message Types which are allways WARN (max messages will count these nevertheless)",
                    orientation='horizontal',
                    allow_empty=False,
                    valuespec=TextInput(allow_empty=True, regex='[a-zA-Z0-9\.]'),
                )
            ),
            (
                "msg_crit",
                ListOfStrings(
                    title=_("CRIT message types"),
                    help = "Message Types which are allways CRIT (max messages will count these nevertheless)",
                    orientation='horizontal',
                    allow_empty=False,
                    valuespec=TextInput(allow_empty=True, regex='[a-zA-Z0-9\.]'),
                )
            ),
            ("levels_upper", Tuple(title=_("max messages"), help = "threshold for a maximum in total messages (counts all including OK/WARN/CRIT and undefined messages types. If you do not want a warn set it equal crit", elements=[
                (Integer(title=_("warn if sum of messages above"))),
                (Integer(title=_("crit if sum of messages above"))),
            ])),
        ]
    )
)


##########################################################################################
##### wato rule definition

def _valuespec_pihole():
    return Dictionary(
        help = _("Pi-hole API Services, this is currently a all in one rule. As there is most likely only one Pi-hole instance running you can either define all your parameters in one rule or only use the desired part of the rule and aim for the coresponding item. "),
        elements = [
            _options_state,
            _options_dbfilesize,
            _options_messages,
            _options_gravity,
        ]
    )


##########################################################################################
##### rule register

rulespec_registry.register(
    CheckParameterRulespecWithItem(
        group=RulespecGroupCheckParametersOperatingSystem,
        check_group_name="pihole",
        item_spec=_item_valuespec_pihole,
        parameter_valuespec=_valuespec_pihole,
        title=lambda: _("Pi-hole API Services"),
    ))    
